@@ -363,6 +363,11 @@ def proxy_options(parser):
         help="Proxy service port."
     )
     http2 = group.add_mutually_exclusive_group()
+    # !!!
+    # Watch out: We raise a RuntimeError in libmproxy.proxy.config if http2 is enabled,
+    # but the OpenSSL version does not have ALPN support (which is the default on Ubuntu 14.04).
+    # Do not simply set --http2 as enabled by default.
+    # !!!
     http2.add_argument("--http2", action="store_true", dest="http2")
     http2.add_argument("--no-http2", action="store_false", dest="http2",
                        help="Explicitly enable/disable experimental HTTP2 support. "
@@ -372,10 +377,10 @@ def proxy_options(parser):
     rawtcp = group.add_mutually_exclusive_group()
     rawtcp.add_argument("--raw-tcp", action="store_true", dest="rawtcp")
     rawtcp.add_argument("--no-raw-tcp", action="store_false", dest="rawtcp",
-                       help="Explicitly enable/disable experimental raw tcp support. "
-                            "Disabled by default. "
-                            "Default value will change in a future version."
-                       )
+                        help="Explicitly enable/disable experimental raw tcp support. "
+                        "Disabled by default. "
+                        "Default value will change in a future version."
+                        )
 
 
 def proxy_ssl_options(parser):
@@ -407,7 +412,7 @@ def proxy_ssl_options(parser):
     group.add_argument(
         "--client-certs", action="store",
         type=str, dest="clientcerts", default=None,
-        help="Client certificate directory."
+        help="Client certificate file or directory."
     )
     group.add_argument(
         "--no-upstream-cert", default=False,
@@ -668,6 +673,11 @@ def mitmproxy():
         action="store_true", dest="eventlog",
         help="Show event log."
     )
+    parser.add_argument(
+        "-f", "--follow",
+        action="store_true", dest="follow",
+        help="Follow flow list."
+    )
     parser.add_argument(
         "--no-mouse",
         action="store_true", dest="no_mouse",
@@ -750,6 +760,21 @@ def mitmweb():
         action="store_true", dest="wdebug",
         help="Turn on mitmweb debugging"
     )
+    group.add_argument(
+        "--wsingleuser",
+        action="store", dest="wsingleuser", type=str,
+        metavar="USER",
+        help="""
+            Allows access to a a single user, specified in the form
+            username:password.
+        """
+    )
+    group.add_argument(
+        "--whtpasswd",
+        action="store", dest="whtpasswd", type=str,
+        metavar="PATH",
+        help="Allow access to users specified in an Apache htpasswd file."
+    )
 
     common_options(parser)
     group = parser.add_argument_group(
